package com.crm.manager.dao.mapper;

import com.crm.rely.backend.core.entity.system.area.SysAreaSearchEntity;
import com.crm.rely.backend.core.pojo.table.SysAreaTable;
import com.crm.rely.backend.core.pojo.view.SysAreaView;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Component;

import java.util.List;

@Component
public interface SysAreaMapper {

    void save(SysAreaTable sysAreaTable);

    void update(SysAreaTable sysAreaTable);

    SysAreaTable findInfoById(Long id);

    Integer countByAreaName(@Param("areaName") String areaName);

    SysAreaView findViewById(Long id);

    Integer countList(SysAreaSearchEntity entity);

    List<SysAreaView> pageList(SysAreaSearchEntity entity);

    void removeById(Long id);

    List<SysAreaTable> findAll();
}
